import { world, system } from "@minecraft/server";
//add scores
world.afterEvents.worldInitialize.subscribe(() => {
    const objectives = ["world", "health", "level", "xpEarnedCurrentLevel", "totalXpNeededToNextLevel", "xpNeededToNextLevel", "x", "y", "z", "disable_message"]
    for (const objective of objectives) {
        world.scoreboard.addObjective(objective, objective)
    }
});
//FUNCTIONS
function getScore(jugador, objetivo) {
    try {
        const score = world.scoreboard.getObjective(objetivo)
        const puntuacion = score.getScore(jugador);
        return puntuacion
    } catch (error) {
        return 0;
    }
};
function applyTagIfTrue(player, propertyName) {
    if (player[propertyName]) {
        player.addTag(propertyName);
    } else {
        player.removeTag(propertyName);
    }
};
function setDimensionTag(player, actualDimension) {
    const dimensionTags = player.getTags().filter(tag => tag.startsWith('dimension:'));
    for (const tag of dimensionTags) {
        if (tag.split(':')[1] !== actualDimension) {
            player.removeTag(tag);
        }
    }
    player.addTag('dimension:' + actualDimension);
}

function getBlockFromViewDirection(player, blockView) {
    const viewTags = player.getTags().filter(tag => tag.startsWith('blockFromViewDirection:'));
    for (const tag of viewTags) {
        if (tag.split(':')[1] !== blockView) {
            player.removeTag(tag);
        }
    }
    player.addTag('blockFromViewDirection:' + blockView);
};

function getEntitiesFromViewDirection(player) {
    const entitiesInView = player.getEntitiesFromViewDirection({ maxDistance: 20 }).map(entity => entity.entity.typeId.replace('minecraft:', ''));

    const entityTags = player.getTags().filter(tag => tag.startsWith('entityFromViewDirection:'));
    for (const tag of entityTags) {
        const entityId = tag.split(':')[1];
        if (!entitiesInView.includes(entityId)) {
            player.removeTag(tag);
        }
    }
    for (const entityInView of entitiesInView) {
        if (!entityTags.includes('entityFromViewDirection:' + entityInView.replace('minecraft:', ''))) {
            player.addTag('entityFromViewDirection:' + entityInView.replace('minecraft:', ''));
        }
    }
};
//ENTITIES 
world.afterEvents.entityHitEntity.subscribe(ev => {
    ev.damagingEntity.addTag('damagingEntity');
    ev.hitEntity.addTag('hitEntity');
    system.runTimeout(() => {
        ev.damagingEntity.removeTag('damagingEntity');
        ev.hitEntity.removeTag('hitEntity');
    }, 1)
});
world.afterEvents.entityHurt.subscribe(ev => {
    ev.hurtEntity.addTag('hurtEntity');
    ev.hurtEntity.addTag('hurtEntity:' + ev.damageSource.cause)
    system.runTimeout(() => {
        const hurt_tags = ev.hurtEntity.getTags().filter(tag => tag.startsWith('hurtEntity'));
        for (const tags of hurt_tags) {
            ev.hurtEntity.removeTag(tags)
        }
    }, 1)
});
world.afterEvents.entityDie.subscribe(ev => {
    if (ev.damageSource.damagingEntity !== undefined) {
        ev.damageSource.damagingEntity.addTag('murderEntity');
        system.runTimeout(() => {
            ev.damageSource.damagingEntity.removeTag('murderEntity');
        }, 1)
    }
    if (ev.deadEntity.typeId !== 'minecraft:player') return;
    ev.deadEntity.addTag('deadEntity');
    system.runTimeout(() => {
        ev.deadEntity.removeTag('deadEntity');
    }, 1)
})
//BLOCKS
world.afterEvents.playerBreakBlock.subscribe(ev => {
    ev.player.addTag('blockBreak');
    ev.player.addTag('blockBreak:' + ev.brokenBlockPermutation.type.id.replace('minecraft:', ''));
    system.runTimeout(() => {
        const blocks_tags = ev.player.getTags().filter(tag => tag.startsWith('blockBreak'));
        for (const block_tag of blocks_tags) {
            ev.player.removeTag(block_tag);
        }
    }, 1)
});
world.afterEvents.playerPlaceBlock.subscribe(ev => {
    ev.player.addTag('blockPlace:' + ev.block.typeId.replace('minecraft:', ''))
    ev.player.addTag('blockPlace');
    system.runTimeout(() => {
        const blocks_tags = ev.player.getTags().filter(tag => tag.startsWith('blockPlace'));
        for (const block_tag of blocks_tags) {
            ev.player.removeTag(block_tag);
        }
    }, 1)
});
world.afterEvents.entityHitBlock.subscribe(ev => {
    ev.damagingEntity.addTag('hitBlock');
    ev.damagingEntity.addTag('hitBlock:' + ev.hitBlock.typeId.replace('minecraft:', ''));
    system.runTimeout(() => {
        const blocks_tags = ev.damagingEntity.getTags().filter(tag => tag.startsWith('hitBlock'));
        for (const block_tag of blocks_tags) {
            ev.damagingEntity.removeTag(block_tag);
        }
    }, 1)
});
//ITEMS
world.afterEvents.itemUse.subscribe(ev => {
    ev.source.addTag('itemUse:' + ev.itemStack.typeId.replace('minecraft:', ''));
    ev.source.addTag('itemUse');
    system.runTimeout(() => {
        const item_tags = ev.source.getTags().filter(tag => tag.startsWith('itemUse'));
        for (const item_tag of item_tags) {
            ev.source.removeTag(item_tag);
        }
    }, 1)
});
//OTHERS
system.runInterval(() => {
    const worldscores = world.scoreboard.getObjective('world');
    const entities = world.getDimension('minecraft:overworld').getEntities();
    const players = entities.filter(plr => plr.typeId === 'minecraft:player');
    const items = entities.filter(item => item.typeId === 'minecraft:item');
    const mobs = entities.filter(mob => mob.typeId !== 'minecraft:player' && mob.typeId !== 'minecraft:item');
    const timeOfDay = world.getTimeOfDay();
    const day = world.getDay();
    worldscores.setScore('day', day)
    worldscores.setScore('timeOfDay', timeOfDay)
    worldscores.setScore('allEntities', entities.length);
    worldscores.setScore('players', players.length);
    worldscores.setScore('items', items.length);
    worldscores.setScore('mobs', mobs.length)
}, 5);
world.afterEvents.weatherChange.subscribe(ev => {
    const weather = world.scoreboard.getObjective('world')
    if (ev.raining) {
        weather.setScore('weatherState', 1)
    }
    else {
        weather.setScore('weatherState', 0)
    }
});
world.beforeEvents.chatSend.subscribe(ev => {
    system.run(() => {
        ev.sender.addTag('chatSend')
        if (ev.message.length < 21) {
            ev.sender.addTag('chatSend:' + ev.message);
        }
    })
    system.runTimeout(() => {
        const item_tags = ev.sender.getTags().filter(tag => tag.startsWith('chatSend'));
        for (const item_tag of item_tags) {
            ev.sender.removeTag(item_tag);
        }
    }, 1)
    const disabled_score = world.scoreboard.getObjective('disable_message');
    const message_list = disabled_score.getParticipants();
    for (const m of message_list) {
        if (ev.message === m.displayName || m.displayName === 'all') {
            ev.cancel = true;
        }
    }
});
//PLAYER
system.runInterval(() => {
    for (const player of world.getPlayers()) {
        const current = player.getComponent('minecraft:health').currentValue
        const health = world.scoreboard.getObjective('health');
        const level = world.scoreboard.getObjective('level');
        const earnedAtCurrentLevel = world.scoreboard.getObjective('xpEarnedCurrentLevel');
        const totalNeededToNextLevel = world.scoreboard.getObjective('totalXpNeededToNextLevel');
        const neededToNextLevel = world.scoreboard.getObjective('xpNeededToNextLevel')
        const x = world.scoreboard.getObjective('x');
        const y = world.scoreboard.getObjective('y');
        const z = world.scoreboard.getObjective('z');
        level.setScore(player, player.level);
        earnedAtCurrentLevel.setScore(player, player.xpEarnedAtCurrentLevel);
        totalNeededToNextLevel.setScore(player, player.totalXpNeededForNextLevel);
        neededToNextLevel.setScore(player, getScore(player, 'totalXpNeededToNextLevel') - getScore(player, 'xpEarnedCurrentLevel'))
        x.setScore(player, player.location.x);
        y.setScore(player, player.location.y);
        z.setScore(player, player.location.z);
        health.setScore(player, current);
        applyTagIfTrue(player, 'isFalling')
        applyTagIfTrue(player, 'isClimbing');
        applyTagIfTrue(player, 'isFlying');
        applyTagIfTrue(player, 'isGliding');
        applyTagIfTrue(player, 'isInWater');
        applyTagIfTrue(player, 'isJumping');
        applyTagIfTrue(player, 'isOnGround');
        applyTagIfTrue(player, 'isSneaking');
        applyTagIfTrue(player, 'isSprinting');
        applyTagIfTrue(player, 'isSwimming');
        setDimensionTag(player, player.dimension.id.replace('minecraft:', ''));
        let blockFromViewDirection = 'air'
        if (player.getBlockFromViewDirection({ maxDistance: 20 }) !== undefined) {
            blockFromViewDirection = player.getBlockFromViewDirection({ maxDistance: 20 }).block.typeId.replace('minecraft:', '');
        }
        getBlockFromViewDirection(player, blockFromViewDirection);
        getEntitiesFromViewDirection(player);

    }
});
world.afterEvents.playerSpawn.subscribe(ev => {
    ev.player.addTag('playerSpawn');
    if (ev.initialSpawn) {
        ev.player.addTag('playerInitialSpawn');
        system.runTimeout(() => {
            ev.player.removeTag('playerInitialSpawn');
        }, 1)
    }
    system.runTimeout(() => {
        ev.player.removeTag('playerSpawn');
    })
});